@@ -2,11 +2,21 @@
 PSD Image module.
 """
 
-from __future__ import absolute_import, unicode_literals
+from __future__ import annotations
 
 import logging
+import os
+from typing import Any, BinaryIO, Callable, Literal
 
-from psd_tools.api import adjustments, deprecated
+try:
+    from typing import Self
+except ImportError:
+    from typing_extensions import Self
+
+import numpy as np
+from PIL.Image import Image as PILImage
+
+from psd_tools.api import adjustments
 from psd_tools.api.layers import (
     Artboard,
     FillLayer,
@@ -17,8 +27,10 @@ from psd_tools.api.layers import (
     SmartObjectLayer,
     TypeLayer,
 )
+from psd_tools.api.pil_io import get_pil_channels, get_pil_mode
 from psd_tools.constants import (
     BlendMode,
+    ChannelID,
     ColorMode,
     CompatibilityMode,
     Compression,
@@ -26,7 +38,17 @@ from psd_tools.constants import (
     SectionDivider,
     Tag,
 )
-from psd_tools.psd import PSD, FileHeader, ImageData, ImageResources
+from psd_tools.psd import (
+    PSD,
+    ChannelImageData,
+    FileHeader,
+    GlobalLayerMaskInfo,
+    ImageData,
+    ImageResources,
+    LayerInfo,
+    LayerRecords,
+    TaggedBlocks,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -48,16 +70,25 @@ class PSDImage(GroupMixin):
             layer_image = layer.compose()
     """
 
-    def __init__(self, data):
+    def __init__(self, data: PSD):
+        from psd_tools.api.layers import Layer
+
         assert isinstance(data, PSD)
         self._record = data
-        self._layers = []
-        self._tagged_blocks = None
+        self._layers: list[Layer] = []
         self._compatibility_mode = CompatibilityMode.DEFAULT
+        self._updated_layers = False  # Flag to check if the layer tree is edited.
         self._init()
 
     @classmethod
-    def new(cls, mode, size, color=0, depth=8, **kwargs):
+    def new(
+        cls,
+        mode: str,
+        size: tuple[int, int],
+        color: int = 0,
+        depth: Literal[8, 16, 32] = 8,
+        **kwargs: Any,
+    ):
         """
         Create a new PSD document.
 
@@ -78,7 +109,7 @@ class PSDImage(GroupMixin):
         )
 
     @classmethod
-    def frompil(cls, image, compression=Compression.RLE):
+    def frompil(cls, image: PILImage, compression=Compression.RLE) -> Self:
         """
         Create a new PSD document from PIL Image.
 
@@ -101,7 +132,7 @@ class PSDImage(GroupMixin):
         )
 
     @classmethod
-    def open(cls, fp, **kwargs):
+    def open(cls, fp: BinaryIO | str | bytes | os.PathLike, **kwargs: Any) -> Self:
         """
         Open a PSD document.
 
@@ -110,29 +141,38 @@ class PSDImage(GroupMixin):
             default 'macroman'. Some psd files need explicit encoding option.
         :return: A :py:class:`~psd_tools.api.psd_image.PSDImage` object.
         """
-        if hasattr(fp, "read"):
-            self = cls(PSD.read(fp, **kwargs))
-        else:
+        if isinstance(fp, (str, bytes, os.PathLike)):
             with open(fp, "rb") as f:
                 self = cls(PSD.read(f, **kwargs))
+        else:
+            self = cls(PSD.read(fp, **kwargs))
         return self
 
-    def save(self, fp, mode="wb", **kwargs):
+    def save(self, fp: BinaryIO | str | bytes | os.PathLike, mode: str = "wb", **kwargs: Any) -> None:
         """
-        Save the PSD file.
+        Save the PSD file. Updates the ImageData section if the layer structure has been updated.
 
         :param fp: filename or file-like object.
         :param encoding: charset encoding of the pascal string within the file,
             default 'macroman'.
         :param mode: file open mode, default 'wb'.
         """
-        if hasattr(fp, "write"):
-            self._record.write(fp, **kwargs)
-        else:
+
+        self._update_record()
+
+        if self._updated_layers:
+            composited_psd = self.composite(force=True)
+            self._record.image_data.set_data([channel.tobytes() for channel in composited_psd.split()], self._record.header)
+
+        if isinstance(fp, (str, bytes, os.PathLike)):
             with open(fp, mode) as f:
                 self._record.write(f, **kwargs)
+        else:
+            self._record.write(fp, **kwargs)
 
-    def topil(self, channel=None, apply_icc=False):
+    def topil(
+        self, channel: int | ChannelID | None = None, apply_icc: bool = True
+    ) -> PILImage | None:
         """
         Get PIL Image.
 
@@ -145,37 +185,15 @@ class PSDImage(GroupMixin):
         """
         from .pil_io import convert_image_data_to_pil
 
+        self._update_record()
+
         if self.has_preview():
             return convert_image_data_to_pil(self, channel, apply_icc)
         return None
 
-    @deprecated
-    def compose(self, force=False, bbox=None, layer_filter=None):
-        """
-        Deprecated, use :py:func:`~psd_tools.PSDImage.composite`.
-
-        Compose the PSD image.
-
-        :param bbox: Viewport tuple (left, top, right, bottom).
-        :return: :py:class:`PIL.Image`, or `None` if there is no pixel.
-        """
-        from psd_tools.composer import compose
-
-        image = None
-        if (not force or len(self) == 0) and not bbox and not layer_filter:
-            image = self.topil()
-        if image is None:
-            image = compose(
-                self,
-                bbox=bbox or self.viewbox,
-                force=force,
-                layer_filter=layer_filter,
-            )
-        elif bbox is not None:
-            image = image.crop(bbox)
-        return image
-
-    def numpy(self, channel=None):
+    def numpy(
+        self, channel: Literal["color", "shape", "alpha", "mask"] | None = None
+    ) -> np.ndarray:
         """
         Get NumPy array of the layer.
 
@@ -189,13 +207,13 @@ class PSDImage(GroupMixin):
 
     def composite(
         self,
-        viewport=None,
-        force=False,
-        color=1.0,
-        alpha=0.0,
-        layer_filter=None,
-        ignore_preview=False,
-        apply_icc=False,
+        viewport: tuple[int, int, int, int] | None = None,
+        force: bool = False,
+        color: float | tuple[float, ...] = 1.0,
+        alpha: float = 0.0,
+        layer_filter: Callable | None = None,
+        ignore_preview: bool = False,
+        apply_icc: bool = True,
     ):
         """
         Composite the PSD image.
@@ -216,13 +234,19 @@ class PSDImage(GroupMixin):
         """
         from psd_tools.composite import composite_pil
 
-        if not (ignore_preview or force or layer_filter) and self.has_preview():
+        self._update_record()
+
+        if (
+            not (ignore_preview or force or layer_filter)
+            and self.has_preview()
+            and not self._updated_layers
+        ):
             return self.topil(apply_icc=apply_icc)
         return composite_pil(
             self, color, alpha, viewport, layer_filter, force, apply_icc=apply_icc
         )
 
-    def is_visible(self):
+    def is_visible(self) -> bool:
         """
         Returns visibility of the element.
 
@@ -231,11 +255,11 @@ class PSDImage(GroupMixin):
         return self.visible
 
     @property
-    def parent(self):
+    def parent(self) -> None:
         """Parent of this layer."""
         return None
 
-    def is_group(self):
+    def is_group(self) -> bool:
         """
         Return True if the layer is a group.
 
@@ -243,7 +267,7 @@ class PSDImage(GroupMixin):
         """
         return isinstance(self, GroupMixin)
 
-    def has_preview(self):
+    def has_preview(self) -> bool:
         """
         Returns if the document has real merged data. When True, `topil()`
         returns pre-composed data.
@@ -254,7 +278,7 @@ class PSDImage(GroupMixin):
         return True  # Assuming the image data is valid by default.
 
     @property
-    def name(self):
+    def name(self) -> str:
         """
         Element name.
 
@@ -263,7 +287,7 @@ class PSDImage(GroupMixin):
         return "Root"
 
     @property
-    def kind(self):
+    def kind(self) -> str:
         """
         Kind.
 
@@ -272,7 +296,7 @@ class PSDImage(GroupMixin):
         return self.__class__.__name__.lower()
 
     @property
-    def visible(self):
+    def visible(self) -> bool:
         """
         Visibility.
 
@@ -281,7 +305,7 @@ class PSDImage(GroupMixin):
         return True
 
     @property
-    def left(self):
+    def left(self) -> int:
         """
         Left coordinate.
 
@@ -290,7 +314,7 @@ class PSDImage(GroupMixin):
         return 0
 
     @property
-    def top(self):
+    def top(self) -> int:
         """
         Top coordinate.
 
@@ -299,7 +323,7 @@ class PSDImage(GroupMixin):
         return 0
 
     @property
-    def right(self):
+    def right(self) -> int:
         """
         Right coordinate.
 
@@ -308,7 +332,7 @@ class PSDImage(GroupMixin):
         return self.width
 
     @property
-    def bottom(self):
+    def bottom(self) -> int:
         """
         Bottom coordinate.
 
@@ -317,7 +341,7 @@ class PSDImage(GroupMixin):
         return self.height
 
     @property
-    def width(self):
+    def width(self) -> int:
         """
         Document width.
 
@@ -326,7 +350,7 @@ class PSDImage(GroupMixin):
         return self._record.header.width
 
     @property
-    def height(self):
+    def height(self) -> int:
         """
         Document height.
 
@@ -335,7 +359,7 @@ class PSDImage(GroupMixin):
         return self._record.header.height
 
     @property
-    def size(self):
+    def size(self) -> tuple[int, int]:
         """
         (width, height) tuple.
 
@@ -344,7 +368,7 @@ class PSDImage(GroupMixin):
         return self.width, self.height
 
     @property
-    def offset(self):
+    def offset(self) -> tuple[int, int]:
         """
         (left, top) tuple.
 
@@ -353,7 +377,7 @@ class PSDImage(GroupMixin):
         return self.left, self.top
 
     @property
-    def bbox(self):
+    def bbox(self) -> tuple[int, int, int, int]:
         """
         Minimal bounding box that contains all the visible layers.
 
@@ -369,7 +393,7 @@ class PSDImage(GroupMixin):
         return bbox
 
     @property
-    def viewbox(self):
+    def viewbox(self) -> tuple[int, int, int, int]:
         """
         Return bounding box of the viewport.
 
@@ -378,7 +402,7 @@ class PSDImage(GroupMixin):
         return self.left, self.top, self.right, self.bottom
 
     @property
-    def color_mode(self):
+    def color_mode(self) -> ColorMode:
         """
         Document color mode, such as 'RGB' or 'GRAYSCALE'. See
         :py:class:`~psd_tools.constants.ColorMode`.
@@ -388,7 +412,7 @@ class PSDImage(GroupMixin):
         return self._record.header.color_mode
 
     @property
-    def channels(self):
+    def channels(self) -> int:
         """
         Number of color channels.
 
@@ -397,7 +421,7 @@ class PSDImage(GroupMixin):
         return self._record.header.channels
 
     @property
-    def depth(self):
+    def depth(self) -> int:
         """
         Pixel depth bits.
 
@@ -406,7 +430,7 @@ class PSDImage(GroupMixin):
         return self._record.header.depth
 
     @property
-    def version(self):
+    def version(self) -> int:
         """
         Document version. PSD file is 1, and PSB file is 2.
 
@@ -415,7 +439,7 @@ class PSDImage(GroupMixin):
         return self._record.header.version
 
     @property
-    def image_resources(self):
+    def image_resources(self) -> ImageResources:
         """
         Document image resources.
         :py:class:`~psd_tools.psd.image_resources.ImageResources` is a
@@ -442,7 +466,7 @@ class PSDImage(GroupMixin):
         return self._record.image_resources
 
     @property
-    def tagged_blocks(self):
+    def tagged_blocks(self) -> TaggedBlocks | None:
         """
         Document tagged blocks that is a dict-like container of settings.
 
@@ -460,7 +484,7 @@ class PSDImage(GroupMixin):
         return self._record.layer_and_mask_information.tagged_blocks
 
     @property
-    def compatibility_mode(self):
+    def compatibility_mode(self) -> CompatibilityMode:
         """
         Set the compositing and layer organization compatibility mode. Writable.
 
@@ -469,18 +493,23 @@ class PSDImage(GroupMixin):
         return self._compatibility_mode
 
     @compatibility_mode.setter
-    def compatibility_mode(self, value):
+    def compatibility_mode(self, value: CompatibilityMode) -> None:
         self._compatibility_mode = value
         self._compute_clipping_layers()
 
-    def has_thumbnail(self):
+    @property
+    def pil_mode(self) -> str:
+        alpha = self.channels - get_pil_channels(get_pil_mode(self.color_mode))
+        return get_pil_mode(self.color_mode, alpha > 0)
+
+    def has_thumbnail(self) -> bool:
         """True if the PSDImage has a thumbnail resource."""
         return (
             Resource.THUMBNAIL_RESOURCE in self.image_resources
             or Resource.THUMBNAIL_RESOURCE_PS4 in self.image_resources
         )
 
-    def thumbnail(self):
+    def thumbnail(self) -> PILImage | None:
         """
         Returns a thumbnail image in PIL.Image. When the file does not
         contain an embedded thumbnail image, returns None.
@@ -493,11 +522,11 @@ class PSDImage(GroupMixin):
             )
         elif Resource.THUMBNAIL_RESOURCE_PS4 in self.image_resources:
             return convert_thumbnail_to_pil(
-                self.image_resources.get_data(Resource.THUMBNAIL_RESOURCE_PS4), "BGR"
+                self.image_resources.get_data(Resource.THUMBNAIL_RESOURCE_PS4)
             )
         return None
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return ("%s(mode=%s size=%dx%d depth=%d channels=%d)") % (
             self.__class__.__name__,
             self.color_mode,
@@ -528,7 +557,9 @@ class PSDImage(GroupMixin):
         _pretty(self, p)
 
     @classmethod
-    def _make_header(cls, mode, size, depth=8):
+    def _make_header(
+        cls, mode: str, size: tuple[int, int], depth: Literal[8, 16, 32] = 8
+    ) -> FileHeader:
         from .pil_io import get_color_mode
 
         assert depth in (8, 16, 32), "Invalid depth: %d" % (depth)
@@ -560,12 +591,14 @@ class PSDImage(GroupMixin):
                         return pattern
         return None
 
-    def _clear_clipping_layers(self):
+    def _clear_clipping_layers(self) -> None:
         for layer in self.descendants():
             layer._clip_layers = []
             layer._has_clip_target = True
 
-    def _compute_clipping_layers(self):
+    def _compute_clipping_layers(self) -> None:
+        self._update_record()
+
         self._clear_clipping_layers()
 
         def rec_helper(layer):
@@ -593,16 +626,18 @@ class PSDImage(GroupMixin):
 
         rec_helper(self)
 
-    def _init(self):
+    def _init(self) -> None:
         """Initialize layer structure."""
-        group_stack = [self]
+        from psd_tools.api.layers import Layer
+
+        group_stack: list[Group | Artboard | PSDImage] = [self]
 
         for record, channels in self._record._iter_layers():
             current_group = group_stack[-1]
 
             blocks = record.tagged_blocks
             end_of_group = False
-            layer = None
+            layer: Layer | Group | Artboard | PSDImage | None = None
             divider = blocks.get_data(Tag.SECTION_DIVIDER_SETTING, None)
             divider = blocks.get_data(Tag.NESTED_SECTION_DIVIDER_SETTING, divider)
             if (
@@ -614,14 +649,20 @@ class PSDImage(GroupMixin):
                 divider.kind is not SectionDivider.OTHER
             ):
                 if divider.kind == SectionDivider.BOUNDING_SECTION_DIVIDER:
-                    layer = Group(self, None, None, current_group)
+                    layer = Group(  # type: ignore
+                        psd=self,
+                        record=None,  # type: ignore
+                        channels=None,  # type: ignore
+                        parent=current_group,
+                    )
+                    layer._set_bounding_records(record, channels)
                     group_stack.append(layer)
                 elif divider.kind in (
                     SectionDivider.OPEN_FOLDER,
                     SectionDivider.CLOSED_FOLDER,
                 ):
                     layer = group_stack.pop()
-                    assert layer is not self
+                    assert not isinstance(layer, PSDImage)
 
                     layer._record = record
                     layer._channels = channels
@@ -669,6 +710,61 @@ class PSDImage(GroupMixin):
             assert layer is not None
 
             if not end_of_group:
+                assert not isinstance(layer, PSDImage)
                 current_group._layers.append(layer)
 
         self._compute_clipping_layers()
+
+    def _update_record(self) -> None:
+        """
+        Compiles the tree layer structure back into records and channels list recursively
+        """
+
+        if not self._updated_layers:
+            # Skip if nothing is changed.
+            return
+
+        layer_records, channel_image_data = _build_record_tree(self)
+
+        # PSDImage.frompil doesn't create a LayerInfo attribute to LayerAndMaskInformation
+        if not self._record.layer_and_mask_information.layer_info:
+            self._record.layer_and_mask_information.layer_info = LayerInfo()
+
+        if not self._record.layer_and_mask_information.global_layer_mask_info:
+            self._record.layer_and_mask_information.global_layer_mask_info = (
+                GlobalLayerMaskInfo()
+            )
+
+        if not self._record.layer_and_mask_information.tagged_blocks:
+            self._record.layer_and_mask_information.tagged_blocks = TaggedBlocks()
+
+        layer_info = self._record.layer_and_mask_information.layer_info
+        layer_info.layer_records = layer_records
+        layer_info.channel_image_data = channel_image_data
+        layer_info.layer_count = len(layer_records)
+
+
+def _build_record_tree(
+    layer_group: GroupMixin,
+) -> tuple[LayerRecords, ChannelImageData]:
+    """
+    Builds the layer tree structure from records and channels list recursively
+    """
+
+    layer_records = LayerRecords()
+    channel_image_data = ChannelImageData()
+
+    for layer in layer_group:
+        if isinstance(layer, (Group, Artboard)):
+            layer_records.append(layer._bounding_record)
+            channel_image_data.append(layer._bounding_channels)
+
+            tmp_layer_records, tmp_channel_image_data = _build_record_tree(layer)
+
+            layer_records.extend(tmp_layer_records)
+            channel_image_data.extend(tmp_channel_image_data)
+
+        layer_records.append(layer._record)
+        channel_image_data.append(layer._channels)
+
+    return (layer_records, channel_image_data)
