@@ -404,8 +404,11 @@ class Tag(bytes, Enum):
     VECTOR_STROKE_CONTENT_DATA = b"vscg"
     VIBRANCE = b"vibA"
 
-    # Unknown
+    # Unknown tags.
     PATT = b"patt"
+    CAI = b"CAI "
+    GENI = b"GenI"
+    OCIO = b"OCIO"
 
 
 TaggedBlockID = Tag  # For backward compatibility.
@@ -515,4 +518,28 @@ class TextType(IntEnum):
     """
 
     POINT = 0
-    PARAGRAPH = 1
\ No newline at end of file
+    PARAGRAPH = 1
+
+
+class ProtectedFlags(IntEnum):
+    """
+    Flags for layer locking
+    """
+    TRANSPARENCY = 0x01
+    COMPOSITE = 0x02
+    POSITION = 0x04
+    NESTING = 0x08
+
+    COMPLETE = 2147483648
+
+    
+class AlphaChannelMode(IntEnum):
+    """
+    alpha channel types.
+    """
+
+    # ref: https://github.com/MolecularMatters/psd_sdk/blob/311b5c2e3fe04c8cc6a563665e66b19b3fcf8116/src/Psd/PsdAlphaChannel.h#L16
+
+    ALPHA = 0  # The channel stores alpha data.
+    INVERTED_ALPHA = 1  # The channel stores inverted alpha data.
+    SPOT = 2  #  The channel stores spot color data.
