@@ -110,6 +110,10 @@ TYPES.update(
         Tag.VECTOR_ORIGINATION_UNKNOWN: IntegerElement,
         Tag.VECTOR_STROKE_DATA: DescriptorBlock,
         Tag.VECTOR_STROKE_CONTENT_DATA: VectorStrokeContentSetting,
+        # Unknown tags.
+        Tag.CAI: DescriptorBlock2,
+        Tag.GENI: DescriptorBlock,
+        Tag.OCIO: DescriptorBlock,
     }
 )
 
